"""Mariner is a command line torrent searcher."""
VERSION = (0, 1, 1)

__title__ = 'mariner'
__description__ = 'Command line torrent searcher'
__url__ = 'http://gitlab.com/radek-sprta/mariner'
__download_url__ = "https://gitlab.com/radek-sprta/mariner/repository/archive.tar.gz?ref=master"
__version__ = '.'.join(map(str, VERSION))
__author__ = 'Radek Sprta'
__author_email__ = 'mail@radeksprta.eu'
__license__ = 'GNU GPLv3'
__copyright__ = 'Copyright 2017 Radek Sprta'
